'''
  $Id: api.py 16 2018-09-13 14:33:18Z dstools $

  AD User Management GUI prototype

  (c) 2018 ITEG IT-Engineers GmbH

'''

# Basic API for user/ group management

import abc
from abc import abstractmethod
import io

class BaseDTO:
    
    def __init__(self,**kwargs):
        for key in self.__slots__:
            setattr(self,key,kwargs.get(key))
    
    def __str__(self):
        
        with io.StringIO() as fd:
        
            fd.write(self.__class__.__name__)
            sep = '['
            
            for key in self.__slots__:
                
                v = getattr(self,key)
                
                if v is not None and len(v) > 32:
                    v = v[:12] + '...' + v [-12:]
                
                fd.write(sep)
                fd.write(key)
                fd.write('=')
                fd.write(str(v))
                sep = ','
            
            fd.write(']')
            return fd.getvalue()
    
    def __repr__(self):
        return self.__str__()

class User(BaseDTO):
    
    __slots__ = ("username","prettyname","email","mobile","yubikey_id","groups","ssh_pubkey","pgp_key")

class Group(BaseDTO):
    __slots__ = ("groupname","label")


class UserService (metaclass=abc.ABCMeta):
    
    @abstractmethod
    def getAllGroups(self):
        raise NotImplementedError()

    @abstractmethod
    def getAllUsers(self):
        raise NotImplementedError()
    
    @abstractmethod
    def getUsersInGroup(self,group):
        raise NotImplementedError()
    
    @abstractmethod
    def createUser(self,user):
        raise NotImplementedError()

    @abstractmethod
    def updateUser(self,user):
        raise NotImplementedError()

    @abstractmethod
    def deleteUser(self,username):
        raise NotImplementedError()

    @abstractmethod
    def findUser(self,username):
        raise NotImplementedError()
    
    @abstractmethod
    def setUserPassword(self,username,passwd):
        raise NotImplementedError()