'''
  $Id$

  AD User Management GUI prototype

  (c) 2018 ITEG IT-Engineers GmbH

'''

from gi.repository import Gtk
from gi.repository import Gio
from gi.repository.Gtk import Dialog
from userproto.api import User

class EditDialog(Dialog):
    def __init__(self,parent,user=None,groups=[]):
        
        self.open = False
        self.runing = False
        self.groups = groups
        self.reset_listbox_idle = True
        
        if user is None:
            title = "Creating new User"
        else:
            title = "Editing User %s"%user.username
            
        Gtk.Dialog.__init__(self, title, parent, 0,
        (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
             Gtk.STOCK_SAVE, Gtk.ResponseType.OK))
        
        self.user = User() if user is None else user
        self.editmode = user is not None
        
        paned = Gtk.VPaned()
        grid = Gtk.Grid()
        
        self.usernamewidget = Gtk.Entry()
        self.usernamewidget.set_editable(not self.editmode)
        self.usernamewidget.connect("notify::text",self.check_yubiinit)
        self.usernamewidget.connect("notify::text",self.check_save)
        if user:
            self.usernamewidget.set_text(user.username)
        usernamelbl = Gtk.Label("User Name:")
        usernamelbl.set_halign(Gtk.Align.START)
        grid.attach(usernamelbl,1,1,1,1)
        self.usernamewidget.set_hexpand(True)
        grid.attach(self.usernamewidget,2,1,1,1)
        
        self.groupswidget = Gtk.ListBox()
        self.groupswidget.set_selection_mode(Gtk.SelectionMode.MULTIPLE)
        for group in groups:
            self.groupswidget.prepend(Gtk.Label(group.label))
        if self.user.groups:
            for group in user.groups:
                for index,i in enumerate(map(lambda x:x.groupname,groups)):
                    if i==group:
                        break
                else:
                    raise ValueError("group [%s] not in groups [%s]"%(group,groups))
                self.groupswidget.select_row(self.groupswidget.get_row_at_index(len(groups)-1-index))
        self.selectedrows = self.groupswidget.get_selected_rows()
        self.groupswidget.connect("selected-rows-changed",self.reset_listbox)
        self.groupswidget.connect("selected-rows-changed",self.check_save)
        self.groupswidget.set_activate_on_single_click(False)
        sw = Gtk.ScrolledWindow()
        sw.set_vexpand(True)
        sw.add(self.groupswidget)
        grid.attach(sw,1,2,2,1)
        
        self.prettynamewidget = Gtk.Entry()
        self.prettynamewidget.set_editable(not self.editmode)
        self.prettynamewidget.connect("notify::text",self.check_yubiinit)
        self.prettynamewidget.connect("notify::text",self.check_save)
        if user:
            self.prettynamewidget.set_text(user.prettyname)
        prettynamelbl = Gtk.Label("Pretty Name:")
        prettynamelbl.set_halign(Gtk.Align.START)
        grid.attach(prettynamelbl,1,3,1,1)
        self.prettynamewidget.set_hexpand(True)
        grid.attach(self.prettynamewidget,2,3,1,1)
        
        self.emailwidget = Gtk.Entry()
        self.emailwidget.set_editable(not self.editmode)
        self.emailwidget.connect("notify::text",self.check_yubiinit)
        self.emailwidget.connect("notify::text",self.check_save)
        if user:
            self.emailwidget.set_text(user.email)
        emaillbl = Gtk.Label("E-Mail:")
        emaillbl.set_halign(Gtk.Align.START)
        grid.attach(emaillbl,1,4,1,1)
        self.emailwidget.set_hexpand(True)
        grid.attach(self.emailwidget,2,4,1,1)
        
        self.mobilewidget = Gtk.Entry()
        self.mobilewidget.set_editable(not self.editmode)
        self.mobilewidget.connect("notify::text",self.check_save)
        if user:
            self.mobilewidget.set_text(user.mobile)
        mobilelbl = Gtk.Label("Mobile:")
        mobilelbl.set_halign(Gtk.Align.START)
        grid.attach(mobilelbl,1,5,1,1)
        self.mobilewidget.set_hexpand(True)
        grid.attach(self.mobilewidget,2,5,1,1)
        
        self.yubiinitbutton =  Gtk.Button.new_with_label("Initialize YubiKey")
        self.yubiinitbutton.set_sensitive(False)
        grid.attach(self.yubiinitbutton,2,6,1,1)
        
        self.yubikeyidwidget = Gtk.Entry()
        self.yubikeyidwidget.set_editable(not self.editmode)
        if user:
            self.yubikeyidwidget.set_text(user.yubikey_id)
        yubikeyidlabel = Gtk.Label("Yubikey Id:")
        yubikeyidlabel.set_halign(Gtk.Align.START)
        grid.attach(yubikeyidlabel,1,7,1,1)
        self.yubikeyidwidget.set_hexpand(True)
        grid.attach(self.yubikeyidwidget,2,7,1,1)
        
        self.sshpupkeywidget = Gtk.Entry()
        self.sshpupkeywidget.set_editable(not self.editmode)
        if user:
            self.sshpupkeywidget.set_text(user.ssh_pubkey)
        sshpupkeylabel = Gtk.Label("SSH Pubkey:")
        sshpupkeylabel.set_halign(Gtk.Align.START)
        grid.attach(sshpupkeylabel,1,8,1,1)
        self.sshpupkeywidget.set_hexpand(True)
        grid.attach(self.sshpupkeywidget,2,8,1,1)
        
        paned.pack1(grid, True, False)
        
        grid2 = Gtk.Grid()
        
        pgpkeylabel = Gtk.Label("PGP Key")
        pgpkeylabel.set_hexpand(True)
        grid2.attach(pgpkeylabel,1,1,2,1)
        
        self.pgpkeywidget = Gtk.TextView()
        self.pgpkeywidget.set_editable(not self.editmode)
        tb = Gtk.TextBuffer()
        if user:
            tb.set_text(user.pgp_key)
        self.pgpkeywidget.set_buffer(tb)
        sw = Gtk.ScrolledWindow()
        sw.set_vexpand(True)
        sw.set_hexpand(True)
        sw.add(self.pgpkeywidget)
        grid2.attach(sw,1,2,2,1)
        
        if self.editmode:
            self.open = False
            self.lockbutton = Gtk.ToggleButton()
            box = Gtk.HBox()
            box.set_hexpand(False)
            self.lockimage = Gtk.Image.new_from_gicon(Gio.ThemedIcon.new_from_names(['changes-prevent']),1)
            box.pack_start(self.lockimage,False,False,0)
            self.locklabel = Gtk.Label("Unlock")
            box.pack_end(self.locklabel,False,False,0)
            box2 = Gtk.HBox()
            box2.pack_start(box,True,False,0)
            self.lockbutton.add(box2)
            self.lockbutton.connect("toggled",self.lockbutton_toggled)
            #print(self.lockbutton.get_child().get_children()[0].get_gicon()[0].get_names())
            grid2.attach(self.lockbutton,1,3,2,1)
        else:
            self.open = True
            self.lockbutton = None    
        self.runing = True
        
        paned.pack2(grid2, True, False)
        
        self.get_content_area().add(paned)
        self.get_action_area().get_children()[1].set_sensitive(False)
        self.show_all()
        
    def lockbutton_toggled(self,a):  # @UnusedVariable
        self.open = self.lockbutton.get_active()
        if self.lockbutton.get_active() is True:
            self.locklabel.set_label("Lock")
            self.lockimage.set_from_gicon(Gio.ThemedIcon.new_from_names(['changes-allow']),1)
            self.selectedrows = self.groupswidget.get_selected_rows()
            self.usernamewidget.set_editable(not self.editmode)
            self.prettynamewidget.set_editable(True)
            self.emailwidget.set_editable(True)
            self.mobilewidget.set_editable(True)
            self.yubikeyidwidget.set_editable(True)
            self.sshpupkeywidget.set_editable(True)
            self.pgpkeywidget.set_editable(True)
            self.yubiinitbutton.set_sensitive(True)
            self.check_save()
        else:
            self.locklabel.set_label("Unlock")
            self.lockimage.set_from_gicon(Gio.ThemedIcon.new_from_names(['changes-prevent']),1)
            self.usernamewidget.set_editable(False)
            self.prettynamewidget.set_editable(False)
            self.emailwidget.set_editable(False)
            self.mobilewidget.set_editable(False)
            self.yubikeyidwidget.set_editable(False)
            self.sshpupkeywidget.set_editable(False)
            self.pgpkeywidget.set_editable(False)
            self.yubiinitbutton.set_sensitive(False)
            self.get_action_area().get_children()[1].set_sensitive(False)
    def get_user(self):
        b = self.pgpkeywidget.get_buffer()
        return User(username=self.usernamewidget.get_text(),
                            prettyname=self.prettynamewidget.get_text(),
                            email=self.emailwidget.get_text(),
                            mobile=self.mobilewidget.get_text(),
                            yubikey_id=self.yubikeyidwidget.get_text(),
                            ssh_pubkey=self.sshpupkeywidget.get_text(),
                            pgp_key=b.get_text(b.get_start_iter(),b.get_end_iter(),True),
                            groups=map(lambda x:self.groups[len(self.groups)-1-x.get_index()],self.groupswidget.get_selected_rows()))
    def reset_listbox(self,*a):  # @UnusedVariable
        if  self.reset_listbox_idle and not self.open:
            self.reset_listbox_idle = False
            self.groupswidget.unselect_all()
            for row in self.selectedrows:
                self.groupswidget.select_row(row)
            self.reset_listbox_idle = True
    def check_yubiinit(self,*a):  # @UnusedVariable
        if self.open:
            self.yubiinitbutton.set_sensitive(self.usernamewidget.get_text() and self.prettynamewidget.get_text() and self.emailwidget.get_text())
    def check_save(self,*a):  # @UnusedVariable
        if self.runing:
            self.get_action_area().get_children()[1].set_sensitive(self.usernamewidget.get_text() and self.prettynamewidget.get_text() and self.emailwidget.get_text() and self.mobilewidget.get_text() and self.groupswidget.get_selected_rows())
        elif not self.editmode:
            self.get_action_area().get_children()[1].set_sensitive(False)
        else:
            self.get_action_area().get_children()[1].set_sensitive(self.user.username and self.user.prettyname and self.user.email and self.user.mobile and self.user.groups)