'''
  $Id: main.py 16 2018-09-13 14:33:18Z dstools $

  AD User Management GUI prototype

  (c) 2018 ITEG IT-Engineers GmbH

'''

# The real main GUI program to be imported and started after logging setup.
import logging
from userproto.fakeapi import FakeUserService
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
from userproto.lib import get_ListStore
from userproto import dialogs

log = logging.getLogger(__name__)

class Window(Gtk.Window):
    def __init__(self,userservice):
        self.userservice = userservice
        Gtk.Window.__init__(self,title="test")
        box = Gtk.VBox()
        
        # treeview
        # box with treeview and toolbar
        box2 = Gtk.VBox()
        self.current_group = None
        self.users = self.userservice.getAllUsers()
        ls = get_ListStore(map(lambda x:(x.prettyname,x.email),self.users),str,str)
        f = ls.filter_new()
        f.set_visible_func(self.filter_func)
        self.treeview = Gtk.TreeView.new_with_model(f)
        self.treeview.connect("row-activated",self.treeview_row_activated)
        renderer_text1 = Gtk.CellRendererText()
        column_text = Gtk.TreeViewColumn("prettyname", renderer_text1, text=0)
        self.treeview.append_column(column_text)
        renderer_text2 = Gtk.CellRendererText()
        column_text = Gtk.TreeViewColumn("email", renderer_text2, text=1)
        self.treeview.append_column(column_text)
        self.treeview.get_selection().set_mode(Gtk.SelectionMode.BROWSE)
        self.treeview.get_selection().connect("changed",self.check_button_activity)
        sw = Gtk.ScrolledWindow()
        sw.set_vexpand(True)
        sw.set_hexpand(True)
        sw.add(self.treeview)
        box2.pack_start(sw,True,True,0)
        #toolbar
        toolbar = Gtk.Toolbar()
        new_tb = Gtk.ToolButton(icon_widget=Gtk.Image.new_from_icon_name(Gtk.STOCK_ADD,Gtk.IconSize.MENU))
        new_tb.connect("clicked",self.on_new_clicked)
        toolbar.insert(new_tb,-1)
        self.remove_tb = Gtk.ToolButton(icon_widget=Gtk.Image.new_from_icon_name(Gtk.STOCK_REMOVE,Gtk.IconSize.MENU))
        self.remove_tb.connect("clicked",self.on_remove_clicked)
        self.remove_tb.set_sensitive(False)
        toolbar.insert(self.remove_tb,-1)
        self.edit_tb = Gtk.ToolButton(icon_widget=Gtk.Image.new_from_icon_name(Gtk.STOCK_EDIT,Gtk.IconSize.MENU))
        self.edit_tb.connect("clicked",self.on_edit_clicked)
        self.edit_tb.set_sensitive(False)
        toolbar.insert(self.edit_tb,-1)
        box2.pack_end(toolbar,False,False,0)
        box.pack_end(box2,True,True,0)
        
        # combobox
        self.groups = self.userservice.getAllGroups()
        ls = get_ListStore(map(lambda x:x.label,self.groups),str)
        self.groups.insert(0,None)
        ls.prepend(["all"])
        self.combobox = Gtk.ComboBox.new_with_model(ls)
        self.combobox.connect("changed",self.combo_box_changed)
        renderer_text = Gtk.CellRendererText()
        self.combobox.set_active(0)
        self.combobox.pack_start(renderer_text, True)
        self.combobox.add_attribute(renderer_text, "text", 0)
        self.combobox.set_hexpand(True)
        
        hbox = Gtk.HBox()
        hbox.pack_start(Gtk.Label("Group"),False,False,0)
        hbox.pack_end(self.combobox,True,True,0)
        
        
        box.pack_start(hbox,False,False,0)
        
        self.add(box)
    def on_remove_clicked(self,*a):  # @UnusedVariable
        
        selected_rows = self.treeview.get_selection().get_selected_rows()
        
        if len(selected_rows[1]) < 1:
            return
        
        path = selected_rows[1][0]
        username = [user.username for user in self.users if ((self.current_group.groupname in user.groups) if self.current_group else True)][path.get_indices()[0]]
        self.userservice.deleteUser(username)
        self.reset()
    def on_new_clicked(self,*a):  # @UnusedVariable
        dialog = dialogs.EditDialog(self,groups=self.groups[1:])
        response = dialog.run()
        if response==Gtk.ResponseType.OK:
            user = dialog.get_user()
            dialog.destroy()
            self.userservice.createUser(user)
            self.reset()
        else:
            dialog.destroy()
    
    def check_button_activity(self,*a): # @UnusedVariable
        selected_rows = self.treeview.get_selection().get_selected_rows()
        
        active = len(selected_rows[1]) >= 1
        self.edit_tb.set_sensitive(active)
        self.remove_tb.set_sensitive(active)
    
    def on_edit_clicked(self,*a):  # @UnusedVariable
        
        selected_rows = self.treeview.get_selection().get_selected_rows()
        
        if len(selected_rows[1]) < 1:
            return
        
        path = selected_rows[1][0]
        user = [u for u in self.users if ((self.current_group.groupname in u.groups) if self.current_group else True)][path.get_indices()[0]]
        self.edit_user(user)

    def treeview_row_activated (self,treeview,path,column):  # @UnusedVariable
        user = [u for u in self.users if ((self.current_group.groupname in u.groups) if self.current_group else True)][path.get_indices()[0]]
        self.edit_user(user)

    def edit_user (self,user):  # @UnusedVariable
        dialog = dialogs.EditDialog(self,user,self.groups[1:])
        response = dialog.run()
        if response==Gtk.ResponseType.OK:
            user = dialog.get_user()
            dialog.destroy()
            self.userservice.updateUser(user)
            self.reset()
        else:
            dialog.destroy()
    def reset(self):
        self.users = self.userservice.getAllUsers()
        liststore = self.treeview.get_model().get_model()
        liststore.clear()
        get_ListStore(map(lambda x:(x.prettyname,x.email),self.users),liststore=liststore)
        
        
    def combo_box_changed(self,combobox):
        self.current_group = self.groups[combobox.get_model()[combobox.get_active_iter()].path.get_indices()[0]]
        self.treeview.get_model().refilter()
    def filter_func(self,liststore,treeiter,a):  # @UnusedVariable
        if self.current_group:
            return self.current_group.groupname in self.users[liststore[treeiter].path.get_indices()[0]].groups
        return True

def gui_main(argv0,args):
    
    log.info("Main called as [%s] with argv [%s]"%(argv0,args))
    
    userService = FakeUserService()
    
    win = Window(userService)
    win.connect("delete-event", Gtk.main_quit)
    win.show_all()
    Gtk.main()

if __name__=="__main__":
    gui_main([],[])